# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import logging

from django.db import models
from django.utils import timezone
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from . import rapid

logger = logging.getLogger('eway')


class ModificationMixin(models.Model):
    date_created = models.DateTimeField(_("date created"))
    date_modified = models.DateTimeField(_("date modified"))

    def save(self, *args, **kwargs):
        now = timezone.now()
        if not self.date_created:
            self.date_created = now
        self.date_modified = now
        return super(ModificationMixin, self).save(*args, **kwargs)

    class Meta:
        abstract = True


class Transaction(ModificationMixin):
    """
    A representation of a EWay transaction for a specific Oscar order.
    """
    IN_PROGRESS = 'in progress'
    COMPLETED = 'completed'
    SUSPICIOUS = 'suspicious'

    STATUSES = (
        (IN_PROGRESS, _("in progress")),
        (COMPLETED, _("completed")),
        (SUSPICIOUS, _("suspicious"))
    )

    access_code = models.CharField(
        _("access code"), max_length=255, blank=True)
    token_customer_id = models.CharField(
        _("token customer ID"), max_length=16, blank=True, default='')
    transaction_id = models.CharField(
        _("transaction ID"), max_length=100, blank=True, default='')

    amount = models.DecimalField(
        _("amount"), decimal_places=2, max_digits=12, blank=True, null=True)

    basket = models.ForeignKey(
        'basket.Basket', verbose_name=_("basket"),
        related_name='eway_transactions', null=True, on_delete=models.SET_NULL)

    # we use this to store the order number in case an order is deleted. This
    # is automatically populated when saving the model
    order_number = models.CharField(
        _("order number"), max_length=255, blank=True, default='')

    status = models.CharField(
        _("status"), max_length=255, default=IN_PROGRESS, choices=STATUSES)

    @cached_property
    def is_completed(self):
        return self.status == self.COMPLETED

    @cached_property
    def last_request_log(self):
        logs = self.request_logs.all()[:1]
        if len(logs):
            return logs[0]
        return None

    @cached_property
    def is_approved(self):
        log = self.last_request_log
        if not log:
            return False
        return log.response_code == rapid.TRANSACTION_APPROVED

    @cached_property
    def is_rejected(self):
        return not self.is_approved

    @cached_property
    def response_code(self):
        try:
            return self.last_request_log.response_code
        except AttributeError:
            return ''

    @cached_property
    def response_message(self):
        try:
            return self.last_request_log.response_message
        except AttributeError:
            return ''

    @cached_property
    def order(self):
        if not self.order_number:
            return

        Order = models.get_model('order', 'Order')
        try:
            return Order.objects.get(number=self.order_number)
        except Order.DoesNotExist:
            logger.info(
                "could not find order with number '{0}', not setting order "
                "on transaction: {1}".format(self.order_number, unicode(self)))
            return None

    def __unicode__(self):
        txn_id = self.transaction_id or self.access_code
        return "Transaction {0}".format(txn_id[:30])

    class Meta:
        ordering = ('-date_created',)
        verbose_name = _("transaction")
        verbose_name_plural = _("transactions")


class RequestLog(ModificationMixin):
    """
    Log entry for requests submitted to the EWay API.
    """
    transaction = models.ForeignKey(
        'eway.Transaction', verbose_name=_("transaction"),
        related_name="request_logs")

    url = models.TextField(_("request URL"), default='')
    method = models.CharField(_("request_method"), max_length=255, blank=True)

    request = models.TextField(_("request message"), blank=True)
    response = models.TextField(_("response message"), blank=True, default='')

    response_code = models.CharField(
        _("response code"), max_length=2, blank=True, default='')
    response_message = models.CharField(
        _("response message"), max_length=255, blank=True, default='')

    errors = models.TextField(_("errors"), blank=True, default='')

    def __unicode__(self):
        return "Request '{0}' for TXN {1}".format(
            self.method, self.transaction.transaction_id)

    class Meta:
        ordering = ('-date_created',)
        verbose_name = _("request log")
        verbose_name_plural = _("request logs")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin
from django.db.models import get_model


class RequestLogInline(admin.TabularInline):
    model = get_model('eway', 'RequestLog')
    readonly_fields = (
        'method', 'response_code', 'response_message',
        'errors', 'date_created', 'date_modified')
    exclude = ('url', 'request', 'response')
    extra = 0
    can_delete = False


class TransactionAdmin(admin.ModelAdmin):
    search_fields = (
        'access_code', 'transaction_id', 'token_customer_id', 'order_number')
    list_display = (
        'access_code', 'transaction_id', 'amount', 'token_customer_id',
        'order_number')

    inlines = [RequestLogInline]


class RequestLogAdmin(admin.ModelAdmin):
    search_fields = ('request', 'response')
    list_display = (
        'transaction', 'method', 'url', 'response_code', 'response_message')
    readonly_fields = (
        'transaction', 'url', 'method', 'request', 'response', 'response_code',
        'response_message', 'errors', 'date_created', 'date_modified')


admin.site.register(get_model('eway', 'Transaction'), TransactionAdmin)
admin.site.register(get_model('eway', 'RequestLog'), RequestLogAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from django.views import generic
from django.db.models import get_model

Transaction = get_model('eway', 'Transaction')


class TransactionListView(generic.ListView):
    model = Transaction
    context_object_name = 'transactions'
    template_name = 'eway/dashboard/transaction_list.html'
    paginate_by = 20


class TransactionDetailView(generic.DetailView):
    model = Transaction
    context_object_name = 'transaction'
    template_name = 'eway/dashboard/transaction_detail.html'



from datetime import date

from django import forms
from django.db.models import get_model
from django.utils.translation import ugettext_lazy as _

from oscar.apps.payment import forms as payment_forms

BankcardModel = get_model('payment', 'Bankcard')

try:
    from oscar.apps.payment.bankcards import bankcard_type
    Bankcard = BankcardModel
except ImportError:
    from oscar.apps.payment.forms import bankcard_type
    from oscar.core.loading import get_class
    Bankcard = get_class('payment.utils', 'Bankcard')


def expiry_month_choices():
    """
    Get a list of numeric month as tuples (*numeric month*, *numeric month*).
    The months are numbered one through 12 according to ISO standard.
    """
    return [("%.2d" % x, "%.2d" % x) for x in xrange(1, 13)]


def expiry_year_choices(num_years=5):
    """
    Get a list of years starting with the current year and going
    *num_years* into the future. The returned list contains
    tuples of the form (*numeric year*, *numeric lyear*) with years
    in 4-digit notation, e.g. 2016.
    """
    return [(x, x) for x in xrange(date.today().year,
                                   date.today().year + num_years)]


def start_month_choices():
    """
    Get a list of numeric month as tuples (*numeric month*, *numeric month*).
    The months are numbered one through 12 according to ISO standard. The first
    item in the list is '--' to allow for not selecting a month.
    """
    return [("", _("--"))] + expiry_month_choices()


def start_year_choices(num_years=5):
    """
    Get a list of years for the starting date of a bank card with the
    range of years  starting *num_years* ago ranging up to the current
    year. The returned list contains tuples of the form (*numeric year*,
    *numeric year*) in 4-digit notation, e.g. 2016. The first item in the
    list is '--' to allow for not selecting a year.
    """
    years = [("", _("--"))]
    for year in xrange(date.today().year - num_years, date.today().year + 1):
        years.append((year, year))
    return years


class BankcardForm(forms.Form):
    """
    A form specific for eWay as specified in the Rapid 3.0 API documentation.
    Submitting the details directly to eWay requires the form's input field
    to have the exact names specified in the API docs. The field names in this
    form therefore violate PEP8 knowingly to remain explicit and not introduce
    too much magic.
    """
    token_customer_id = forms.ChoiceField(
        required=False,
        label=_("Existing cards"),
        widget=forms.RadioSelect(),
    )

    EWAY_ACCESSCODE = forms.CharField(
        label=_("Access code"),
        widget=forms.HiddenInput(),
        required=False,
    )
    EWAY_CARDNAME = forms.CharField(
        required=False,
        label=_("Name"),
        help_text=_("Your name as it appears on the card")
    )
    EWAY_CARDNUMBER = payment_forms.BankcardNumberField(
        max_length=20,
        required=False,
        widget=forms.TextInput(attrs={'autocomplete': 'off'}),
        label=_("Card number")
    )
    EWAY_CARDCVN = forms.RegexField(
        required=True,
        label=_("CVV Number"),
        regex=r'^\d{3,4}$',
        widget=forms.TextInput(attrs={'size': '5'}),
        help_text=_("This is the 3 or 4 digit security number on the back of "
                    "your bankcard")
    )

    EWAY_CARDEXPIRYMONTH = forms.ChoiceField(
        choices=expiry_month_choices(),
        label=_("Valid to"),
        required=False
    )
    EWAY_CARDEXPIRYYEAR = forms.ChoiceField(
        choices=expiry_year_choices(6),
        label="",
        required=False
    )

    # these fields are used in the UK only
    EWAY_CARDSTARTMONTH = forms.ChoiceField(
        choices=start_month_choices(),
        widget=forms.HiddenInput(),
        required=False
    )
    EWAY_CARDSTARTYEAR = forms.ChoiceField(
        choices=start_year_choices(),
        widget=forms.HiddenInput(),
        required=False
    )
    EWAY_CARDISSUENUMBER = forms.CharField(
        widget=forms.HiddenInput(),
        required=False
    )

    def __init__(self, *args, **kwargs):
        user = kwargs.pop('user', None)
        is_hidden = kwargs.pop('is_hidden', None)

        super(BankcardForm, self).__init__(*args, **kwargs)

        if user and user.is_authenticated():
            choices = self.get_existing_cards(user)
            self.fields['token_customer_id'].choices = choices
        else:
            self.fields['token_customer_id'].widget = forms.HiddenInput()

        if is_hidden:
            for fname in self.fields:
                self.fields[fname].widget = forms.HiddenInput()

    def get_existing_cards(self, user):
        """
        Get a list of tuples for existing cards of the specified *user*.
        Each tuple contains the ``token_customer_id`` and the obfuscated
        bankcard number.
        """
        existing_cards = BankcardModel.objects.filter(
            user=user,
            partner_reference__isnull=False
        )
        return [(c.partner_reference, c.number) for c in existing_cards]

    def check_is_required(self, field_name):
        """
        Check if the field with *field_name* is required based on the
        selection of a ``token_customer_id`` indicating the use of an
        existing payment.
        A field is required if no ``token_customer_id`` is provided
        and having the field *field_name* empty will raise a
        ``ValidationError``.
        """
        data = self.cleaned_data[field_name]
        if not data and not self.cleaned_data['token_customer_id']:
            raise forms.ValidationError(_("This field is required"))
        return data

    def clean_EWAY_CARDNAME(self):
        return self.check_is_required('EWAY_CARDNAME')

    def clean_EWAY_CARDNUMBER(self):
        return self.check_is_required('EWAY_CARDNUMBER')

    def clean_EWAY_CARDEXPIRYMONTH(self):
        return self.check_is_required('EWAY_CARDEXPIRYMONTH')

    def clean_EWAY_CARDEXPIRYYEAR(self):
        return self.check_is_required('EWAY_CARDEXPIRYYEAR')

    def get_obfuscated_kwargs(self):
        """
        Get a dictionary containing the obfuscated bank card details used
        in the eway form. The card number is **not** stored in plain
        text but is obfuscated in the same way eWay obfuscates their card
        numers (first 6 and last 4 digits visible).
        """
        card_number = self.data.get('EWAY_CARDNUMBER', '')
        return {
            'card_type': bankcard_type(card_number) or '',
            'name': self.data.get('EWAY_CARDNAME', ''),
            'number': self.get_obfuscated_card_number(card_number),
            'expiry_date': "%s/%s" % (
                self.data.get('EWAY_CARDEXPIRYMONTH', ''),
                self.data.get('EWAY_CARDEXPIRYYEAR', ''),
            ),
        }

    @classmethod
    def get_obfuscated_card_number(cls, number):
        """
        Get an obfuscated version of the bankcard *number* with several
        digits replaced with 'X'. We obfuscate using the same method as
        eWay uses which shows the first six and the last four digits. For
        a credit card '4444333322221111' this method returns
        '444433XXXXXX1111'. If *number* has less than 11 digits, the number
        is obfuscated completely to avoid accidentally storing a complete
        bankcard number.
        """
        # if the number is shorter than 10 digits the whole number would
        # show up, in this special case we obfuscate every digit just to
        # be sure that we don't accidentally store actual credit card data
        if len(number) < 11:
            return len(number) * 'X'

        # eway uses 10 visible digits in their obfuscation, we replicate
        # this here to stay compliant
        start, masked, end = number[:6], number[6:-4], number[-4:]
        return "".join([start, len(masked) * 'X', end])

    def get_bankcard_obj(self):
        """
        Returns a Bankcard object for use in payment processing.
        """
        kwargs = {
            'name': self.cleaned_data['EWAY_CARDNAME'],
            'card_number': self.cleaned_data['EWAY_CARDNUMBER'],
            'expiry_date': "%s/%s" % (
                self.cleaned_data['EWAY_CARDEXPIRYMONTH'],
                self.cleaned_data['EWAY_CARDEXPIRYYEAR'],
            ),
            'cvv': self.cleaned_data['EWAY_CARDCVN'],
        }
        if (self.cleaned_data['EWAY_CARDSTARTMONTH']
           and self.cleaned_data['EWAY_CARDSTARTMONTH']):
            kwargs['start_date'] = "%s/%s" % (
                self.cleaned_data['EWAY_CARDSTARTMONTH'],
                self.cleaned_data['EWAY_CARDSTARTMONTH'])
        return Bankcard(**kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import logging

from datetime import timedelta
from optparse import make_option

from django.core.management.base import BaseCommand

from ... import utils

logger = logging.getLogger('eway')


class Command(BaseCommand):
    help = "Check dangling transaction for being suspicious."

    option_list = BaseCommand.option_list + (
        make_option(
            '--minimum-age',
            dest='minimum_age',
            type=int,
            help='Minimum age of transactions to be considered dangling.'),)

    def handle(self, *args, **options):
        minimum_age = options.get('minimum_age')

        if minimum_age:
            logger.info("Using minimum age: {0} hours".format(minimum_age))
            minimum_age = timedelta(hours=minimum_age)

        utils.check_dangling_transactions(minimum_age=minimum_age)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import logging

from datetime import timedelta

from django.utils.timezone import now
from django.db.models import get_model

from .rapid import facade

Transaction = get_model('eway', 'Transaction')

logger = logging.getLogger('eway')
eway_gw = facade.Facade()


def check_dangling_transactions(minimum_age=None):
    """
    Check for transactions that are currently in progress and are at least
    of *minimum_age*. If no minimum age is specified, a default timedelta
    of **2 hours** is used.

    In the process, the transaction might be updated with data retrieved from
    eWay. In addition, suspicious transactions are marked as such in the
    transaction status.

    :param minimum_age: Minimum age of dangling transaction as timedelta.
    :type minimum_age: timedelta or None
    """
    if not minimum_age:
        minimum_age = timedelta(hours=2)

    cutoff_date = now() - minimum_age

    logger.info("Looking up all eway transactions started before: {0}".format(
        cutoff_date.isoformat()))

    #get dangling transactions
    transactions = Transaction.objects.filter(
        date_created__lte=cutoff_date, status=Transaction.IN_PROGRESS)

    logger.info("Found {0} transactions that are in progress".format(
        len(transactions)))

    # check for suspicious transactions and mark them as such
    for transaction in transactions:
        if is_transaction_suspicious(transaction):
            transaction.status = transaction.SUSPICIOUS
            transaction.save()


def is_transaction_suspicious(transaction, check_eway=True):
    """
    Check if the *transaction* is suspicious. A transaction is considered to
    be suspicious if it doesn't have an **access code** assigned to it or if
    its order number doesn't correspond to an order in the system.

    :param transaction: an instance of the eway ``Transaction`` model.
    :returns: ``True`` if transaction is suspicious, ``False`` otherwise.
    """
    if not transaction.access_code:
        logger.error('could not find access code in transaction with '
                     'PK {0}'.format(transaction.id))
        return True

    if check_eway:
        # this updates the transaction with the latest details
        # retrieved from eWAY so we don't have to care about the
        # returned result
        eway_gw.get_access_code_result(transaction.access_code)

    if not transaction.order_number:
        logger.info(
            "ignoring transaction with PK {0}, no order number "
            "generated".format(transaction.id))
        return False

    if transaction.is_rejected and not transaction.order:
        logger.info(
            "transaction {0} was rejected and no order was generated. All "
            "good.".format(transaction.id))
        return False

    if transaction.is_approved and transaction.order:
        logger.info(
            "ignoring transaction with PK {0}, seems to be valid ".format(
                transaction.id))
        return False

    logger.error('transaction with PK {0} looks suspecious, flagging it '
                 'as suspicious.'.format(transaction.id))
    return True



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'EwayTransaction'
        db.create_table('eway_ewaytransaction', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('order_number', self.gf('django.db.models.fields.CharField')(max_length=128, null=True, db_index=True)),
            ('token_customer_id', self.gf('django.db.models.fields.CharField')(max_length=16, null=True, db_index=True)),
            ('txn_url', self.gf('django.db.models.fields.CharField')(max_length=800, null=True)),
            ('txn_method', self.gf('django.db.models.fields.CharField')(max_length=12, null=True)),
            ('txn_ref', self.gf('django.db.models.fields.CharField')(max_length=16, null=True)),
            ('amount', self.gf('django.db.models.fields.DecimalField')(null=True, max_digits=12, decimal_places=2, blank=True)),
            ('response_code', self.gf('django.db.models.fields.CharField')(max_length=2, null=True)),
            ('response_message', self.gf('django.db.models.fields.CharField')(max_length=255, null=True)),
            ('request_json', self.gf('django.db.models.fields.TextField')()),
            ('response_json', self.gf('django.db.models.fields.TextField')(null=True)),
            ('date_created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('eway', ['EwayTransaction'])

        # Adding model 'EwayResponseCode'
        db.create_table('eway_ewayresponsecode', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('code', self.gf('django.db.models.fields.CharField')(unique=True, max_length=10)),
            ('message', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal('eway', ['EwayResponseCode'])

        # Adding M2M table for field transactions on 'EwayResponseCode'
        db.create_table('eway_ewayresponsecode_transactions', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('ewayresponsecode', models.ForeignKey(orm['eway.ewayresponsecode'], null=False)),
            ('ewaytransaction', models.ForeignKey(orm['eway.ewaytransaction'], null=False))
        ))
        db.create_unique('eway_ewayresponsecode_transactions', ['ewayresponsecode_id', 'ewaytransaction_id'])


    def backwards(self, orm):
        # Deleting model 'EwayTransaction'
        db.delete_table('eway_ewaytransaction')

        # Deleting model 'EwayResponseCode'
        db.delete_table('eway_ewayresponsecode')

        # Removing M2M table for field transactions on 'EwayResponseCode'
        db.delete_table('eway_ewayresponsecode_transactions')


    models = {
        'eway.ewayresponsecode': {
            'Meta': {'object_name': 'EwayResponseCode'},
            'code': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '10'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'message': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'transactions': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'response_messages'", 'symmetrical': 'False', 'to': "orm['eway.EwayTransaction']"})
        },
        'eway.ewaytransaction': {
            'Meta': {'ordering': "('-date_created',)", 'object_name': 'EwayTransaction'},
            'amount': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '12', 'decimal_places': '2', 'blank': 'True'}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'order_number': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'db_index': 'True'}),
            'request_json': ('django.db.models.fields.TextField', [], {}),
            'response_code': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True'}),
            'response_json': ('django.db.models.fields.TextField', [], {'null': 'True'}),
            'response_message': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True'}),
            'token_customer_id': ('django.db.models.fields.CharField', [], {'max_length': '16', 'null': 'True', 'db_index': 'True'}),
            'txn_method': ('django.db.models.fields.CharField', [], {'max_length': '12', 'null': 'True'}),
            'txn_ref': ('django.db.models.fields.CharField', [], {'max_length': '16', 'null': 'True'}),
            'txn_url': ('django.db.models.fields.CharField', [], {'max_length': '800', 'null': 'True'})
        }
    }

    complete_apps = ['eway']


# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import json
import logging
import requests

from decimal import Decimal as D

from django.conf import settings
from django.db.models import get_model
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ImproperlyConfigured

from eway.rapid import RESPONSE_CODES

logger = logging.getLogger('eway')

Transaction = get_model('eway', 'Transaction')
RequestLog = get_model('eway', 'RequestLog')


EWAY_PROCESS_PAYMENT = 'ProcessPayment'
EWAY_CREATE_TOKEN_CUSTOMER = 'CreateTokenCustomer'
EWAY_UPDATE_TOKEN_CUSTOMER = 'UpdateTokenCustomer'
EWAY_TOKEN_PAYMENT = 'TokenPayment'

EWAY_PAYMENT_METHODS = (
    EWAY_PROCESS_PAYMENT,
    EWAY_CREATE_TOKEN_CUSTOMER,
    EWAY_UPDATE_TOKEN_CUSTOMER,
    EWAY_TOKEN_PAYMENT,
)

EWAY_TITLE_VALUES = (
    'Mr.',
    'Ms.',
    'Mrs.',
    'Miss',
    'Dr.',
    'Sir.',
    'Prof.',
)

EWAY_SHIPPING_UNKNOWN = 'Unknown'
EWAY_SHIPPING_LOWCOST = 'LowCost'
EWAY_SHIPPING_DESIGNATEDBYCUSTOMER = 'DesignatedByCustomer'
EWAY_SHIPPING_INTERNATIONAL = 'International'
EWAY_SHIPPING_MILITARY = 'Military'
EWAY_SHIPPING_NEXTDAY = 'NextDay'
EWAY_SHIPPING_STOREPICKUP = 'StorePickup'
EWAY_SHIPPING_TWODAYSERVICE = 'TwoDayService'
EWAY_SHIPPING_THREEDAYSERVICE = 'ThreeDayService'
EWAY_SHIPPING_OTHER = 'Other'

EWAY_API_KEY = None
EWAY_PASSWORD = None
EWAY_CURRENCY = 'AUD'


class RapidError(BaseException):
    pass


class RapidResponse(object):

    def __init__(self, access_code, form_action_url, payment, customer, errors,
                 json_raw=None):
        self.access_code = access_code
        self.form_action_url = form_action_url
        self.payment = payment
        self.customer = customer

        self.errors = errors

        # Try to get properly indented JSON, if that fails we fall back to
        # the raw data.
        try:
            self.json_raw = json.dumps(json_raw, indent=4)
        except ValueError:
            self.json_raw = json_raw

    @classmethod
    def from_json(cls, response):
        return cls(
            access_code=response['AccessCode'],
            form_action_url=response['FormActionURL'],
            payment=Payment.from_json(response['Payment']),
            customer=Customer.from_json(response['Customer']),
            errors=cls.extract_errors(response['Errors']),
            json_raw=response)

    @classmethod
    def extract_errors(cls, error_codes):
        if not error_codes:
            return []

        errors = []
        for error_code in error_codes.split(','):
            errors.append(RapidResponseCode(error_code))

        return errors


class RapidResponseCode(object):

    def __init__(self, code):
        self.code = code
        self.message = RESPONSE_CODES.get(code, '')


class TotalAmountMixin(object):

    @property
    def total_amount(self):
        if self.total_amount_raw is None:
            return None
        left = int(self.total_amount_raw / 100)
        right = self.total_amount_raw - int(left * 100)
        return D("%d.%02d" % (left, right))

    @total_amount.setter
    def total_amount(self, value):
        if value is None:
            self.total_amount_raw = None
            return None
        self.total_amount_raw = int(value * 100)
        return value


class RapidAccessCodeResult(TotalAmountMixin):

    def __init__(self, authorisation_code, token_customer_id, access_code,
                 response_message, response_code, transaction_status,
                 transaction_id, total_amount=None, total_amount_raw=None,
                 **kwargs):
        self.total_amount_raw = total_amount_raw

        if self.total_amount_raw is None:
            self.total_amount = total_amount

        self.authorisation_code = authorisation_code
        self.token_customer_id = token_customer_id
        self.access_code = access_code
        self.response_code = response_code
        self.response_message = response_message or []
        self.transaction_status = transaction_status
        self.transaction_id = transaction_id

        self.errors = kwargs.get('errors', [])
        self.verification = kwargs.get('verification', None)
        self.beagle_score = kwargs.get('beagle_score', None)
        self.invoice_reference = kwargs.get('invoice_reference', None)
        self.invoice_number = kwargs.get('invoice_number', None)

        try:
            self.json_raw = json.dumps(kwargs.get('json_raw'), indent=4)
        except ValueError:
            self.json_raw = kwargs.get('json_raw')

    @classmethod
    def from_json(cls, response):
        errors = response.get('Errors', [])
        if errors:
            errors = [RapidResponseCode(c) for c in errors.split(',')]
        else:
            errors = []

        response_message = response.get('ResponseMessage', None)
        if response_message:
            response_message = [RapidResponseCode(c) for c in response_message.split(',')]

        options = []
        for option in response.get('Options', []):
            options.append(Option(option.get("Value", None)))

        verification = Verification.from_json(response.get('Verification', None))

        return cls(
            authorisation_code=response.get('AuthorisationCode', None),
            token_customer_id=response.get('TokenCustomerID', None),
            access_code=response.get('AccessCode', None),
            transaction_status=response.get('TransactionStatus', None),
            response_code=RapidResponseCode(response.get('ResponseCode', None)),
            response_message=response_message,
            verification=verification,
            errors=errors,
            invoice_reference=response.get('InvoiceReference', None),
            total_amount_raw=response.get('TotalAmount', None),
            transaction_id=response.get('TransactionID', None),
            invoice_number=response.get('InvoiceNumber', None),
            options=options,
            json_raw=response
        )


class Verification(object):

    def __init__(self, cvn, mobile, email, phone, address):
        self.cvn = cvn
        self.mobile = mobile
        self.email = email
        self.phone = phone
        self.address = address

    @classmethod
    def from_json(cls, response):
        if response is None:
            return
        return cls(
            cvn=response.get('CVN', None),
            mobile=response.get('Mobile', None),
            email=response.get('Email', None),
            phone=response.get('Phone', None),
            address=response.get('Address', None),
        )


class RapidAttribute(object):

    def __init__(self, name, max_length=None, attr_type=None):
        self.name = name
        self.max_length = max_length
        self.type = attr_type

    def validate(self, value):
        if value is None or self.type in (None, list):
            return

        if len(unicode(value)) > self.max_length:
            raise RapidError(
                _("Attribute value '%s' exceeds maximum length: %d") % (
                    value,
                    self.max_length
                )
            )

    def force_type(self, value):
        return self.type(value)


class RapidBaseObject(object):
    REST_ATTRIBUTES = {}

    def json(self):
        json_dict = {}
        for attr_name, rapid_attr in self.REST_ATTRIBUTES.items():
            attr_value = getattr(self, attr_name, None)

            if attr_value is None and attr_name != 'title':
                continue

            if rapid_attr.type is None:
                json_dict[rapid_attr.name] = attr_value.json()
                continue

            if issubclass(rapid_attr.type, (tuple, list)):
                if not attr_value:
                    continue
                json_dict[rapid_attr.name] = []
                for value in attr_value:
                    json_dict[rapid_attr.name].append(value.json())
                continue

            rapid_attr.validate(attr_value)
            json_dict[rapid_attr.name] = rapid_attr.force_type(attr_value)
        return json_dict

    def serialise(self):
        return json.dumps(self.json())


class Request(RapidBaseObject):
    REST_ATTRIBUTES = {
        'redirect_url': RapidAttribute('RedirectUrl', 512, unicode),
        'customer_ip': RapidAttribute('CustomerIP', 50, unicode),
        'method': RapidAttribute('Method', 20, unicode),
        'device_id': RapidAttribute('DeviceID', 50, unicode),

        'payment': RapidAttribute('Payment'),
        'customer': RapidAttribute('Customer'),
        'shipping_address': RapidAttribute('ShippingAddress'),

        'items': RapidAttribute('Items', None, list),
        'options': RapidAttribute('Options', None, list),
    }

    def __init__(self, redirect_url, method, customer_ip=None, device_id=None,
                 customer=None, payment=None, shipping_address=None, items=None,
                 options=None):

        if method not in EWAY_PAYMENT_METHODS:
            raise RapidError(
                _("payment method %s is not a valid Rapid 3.0 method") % method
            )

        self.redirect_url = redirect_url
        self.method = method
        self.customer_ip = customer_ip or u''
        self.device_id = device_id or u''

        self.payment = payment
        self.customer = customer
        self.shipping_address = shipping_address

        self.items = items or []

        self.options = []
        if options:
            for option in options:
                self.options.append(Option(option))

    def __repr__(self):
        return unicode(self)

    def __unicode__(self):
        return self.serialise()


class Payment(RapidBaseObject, TotalAmountMixin):
    REST_ATTRIBUTES = {
        'total_amount_raw': RapidAttribute('TotalAmount', 10, int),
        'invoice_number': RapidAttribute('InvoiceNumber', 50, unicode),
        'invoice_description': RapidAttribute('InvoiceDescription', 5, unicode),
        'invoice_reference': RapidAttribute('InvoiceReference', 50, unicode),
        'currency_code': RapidAttribute('CurrencyCode', 50, unicode),
    }

    def __init__(self, total_amount=None, invoice_number=None,
                 invoice_description=None, invoice_reference=None,
                 currency_code=None, total_amount_raw=None):

        self.total_amount_raw = total_amount_raw
        if self.total_amount_raw is None:
            self.total_amount = total_amount

        self.invoice_number = invoice_number
        self.invoice_description = invoice_description
        self.invoice_reference = invoice_reference

        if not currency_code:
            currency_code = getattr(settings, 'EWAY_CURRENCY', EWAY_CURRENCY)

        self.currency_code = currency_code

    @classmethod
    def from_json(cls, response):
        return cls(
            total_amount_raw=response.get("TotalAmount", None),
            invoice_number=response.get('InvoiceNumber', None),
            invoice_description=response.get('InvoiceDescription', None),
            invoice_reference=response.get('InvoiceReference', None),
            currency_code=response.get('CurrencyCode', None),
        )


class Customer(RapidBaseObject):
    REST_ATTRIBUTES = {
        'token_customer_id': RapidAttribute('TokenCustomerID', 16, int),
        'reference': RapidAttribute('Reference', 50, unicode),
        'title': RapidAttribute('Title', 5, unicode),
        'first_name': RapidAttribute('FirstName', 50, unicode),
        'last_name': RapidAttribute('LastName', 50, unicode),
        'company_name': RapidAttribute('CompanyName', 50, unicode),
        'job_description': RapidAttribute('JobDescription', 50, unicode),
        'street1': RapidAttribute('Street1', 50, unicode),
        'street2': RapidAttribute('Street2', 50, unicode),
        'city': RapidAttribute('City', 50, unicode),
        'state': RapidAttribute('State', 50, unicode),
        'postal_code': RapidAttribute('PostalCode', 50, unicode),
        'country': RapidAttribute('Country', 2, unicode),
        'email': RapidAttribute('Email', 50, unicode),
        'phone': RapidAttribute('Phone', 32, unicode),
        'mobile': RapidAttribute('Mobile', 32, unicode),
        'comments': RapidAttribute('Comments', 255, unicode),
        'fax': RapidAttribute('Fax', 32, unicode),
        'url': RapidAttribute('Url', 512, unicode),
    }

    def __init__(self, token_customer_id=None, billing_address=None, reference=None,
                 title=None, email=None, phone=None, mobile=None, comments=None,
                 fax=None, url=None, **kwargs):
        self.token_customer_id = token_customer_id  # this is the TokenCustomerID
        self.reference = reference
        self.title = self.get_valid_title(title)
        self.email = email
        self.phone = phone
        self.mobile = mobile
        self.comments = comments
        self.fax = fax
        self.url = url

        self.first_name = kwargs.get('first_name', None)
        self.last_name = kwargs.get('last_name', None)
        self.company_name = kwargs.get('company_name', None)
        self.job_description = kwargs.get('job_description', None)
        self.street1 = kwargs.get('street1', None)
        self.street2 = kwargs.get('street2', None)
        self.city = kwargs.get('city', None)
        self.state = kwargs.get('state', None)
        self.postal_code = kwargs.get('postal_code', None)
        self.country = kwargs.get('country', None)

        self.card_name = kwargs.get('card_name', None)
        self.card_number = kwargs.get('card_number', None)
        self.card_start_month = kwargs.get('card_start_month', None)
        self.card_start_year = kwargs.get('card_start_year', None)
        self.card_issue_number = kwargs.get('card_issue_number', None)
        self.card_expiry_month = kwargs.get('card_expiry_month', None)
        self.card_expiry_year = kwargs.get('card_expiry_year', None)

        if billing_address:
            self.set_address(billing_address)

    def get_valid_title(self, title):
        if title is None:
            return u'Prof.'
        for eway_title in  EWAY_TITLE_VALUES:
            if eway_title.startswith(title):
                return eway_title
        return u'Prof.'

    def set_address(self, address):
        self.title = self.get_valid_title(address.title)

        self.first_name = address.first_name
        self.last_name = address.last_name
        self.street1 = address.line1
        self.street2 = address.line2
        self.city = address.line4
        self.state = address.state
        self.postal_code = address.postcode
        self.country = address.country.iso_3166_1_a2.lower()

    @classmethod
    def from_json(cls, response):
        return cls(
            token_customer_id=response.get("TokenCustomerID", None),
            reference=response.get("Reference", None),
            title=response.get("Title", None),
            first_name=response.get("FirstName", None),
            last_name=response.get("LastName", None),
            company_name=response.get("CompanyName", None),
            job_description=response.get("JobDescription", None),
            street1=response.get("Street1", None),
            street2=response.get("Street2", None),
            city=response.get("City", None),
            state=response.get("State", None),
            postal_code=response.get("PostalCode", None),
            country=response.get("Country", None),
            email=response.get("Email", None),
            phone=response.get("Phone", None),
            mobile=response.get("Mobile", None),
            comments=response.get("Comments", None),
            fax=response.get("Fax", None),
            url=response.get("Url", None),
            card_number=response.get("CardNumber", None),
            card_start_month=response.get("CardStartMonth", None),
            card_start_year=response.get("CardStartYear", None),
            card_issue_number=response.get("CardIssueNumber", None),
            card_name=response.get("CardName", None),
            card_expiry_month=response.get("CardExpiryMonth", None),
            card_expiry_year=response.get("CardExpiryYear", None),
        )


class ShippingAddress(RapidBaseObject):
    REST_ATTRIBUTES = {
        'shipping_method': RapidAttribute('ShippingMethod', 30, unicode),
        'first_name': RapidAttribute('FirstName', 50, unicode),
        'last_name': RapidAttribute('LastName', 50, unicode),
        'street1': RapidAttribute('Street1', 50, unicode),
        'street2': RapidAttribute('Street2', 50, unicode),
        'city': RapidAttribute('City', 50, unicode),
        'state': RapidAttribute('State', 50, unicode),
        'postal_code': RapidAttribute('PostalCode', 50, unicode),
        'country': RapidAttribute('Country', 2, unicode),
        'email': RapidAttribute('Email', 50, unicode),
        'phone': RapidAttribute('Phone', 32, unicode),
        'fax': RapidAttribute('Fax', 32, unicode),
    }

    def __init__(self, shipping_method, shipping_address=None, email=None,
                 phone=None, fax=None):
        self.shipping_method = shipping_method
        self.email = email
        self.phone = phone
        self.fax = fax

        self.first_name = None
        self.last_name = None
        self.street1 = None
        self.street2 = None
        self.city = None
        self.state = None
        self.postal_code = None
        self.country = None

        if shipping_address:
            self.set_shipping_address(shipping_address)

    def set_shipping_address(self, shipping_address):
        self.first_name = shipping_address.first_name
        self.last_name = shipping_address.last_name
        self.street1 = shipping_address.line1
        self.street2 = shipping_address.line2
        self.city = shipping_address.line4
        self.state = shipping_address.state
        self.postal_code = shipping_address.postcode
        self.country = shipping_address.country.iso_3166_1_a2.lower()


class Item(RapidBaseObject):
    REST_ATTRIBUTES = {
        'sku': RapidAttribute('SKU', 12, unicode),
        'description': RapidAttribute('Description', 26, unicode),
        'quantity': RapidAttribute('Quantity', 6, int),
        'unit_cost': RapidAttribute('UnitCost', 8, int),
        'tax': RapidAttribute('Tax', 8, int),
        'total': RapidAttribute('Total', 8, int),
    }

    def __init__(self, sku=None, description=None, quantity=None,
                 unit_cost=None, tax=None, total=None):
        self.sku = sku
        self.description = description
        self.quantity = quantity
        self.unit_cost = unit_cost
        self.tax = tax
        self.total = total


class Option(RapidBaseObject):
    REST_ATTRIBUTES = {
        u'value': RapidAttribute('Value', 254, unicode),
    }

    def __init__(self, value):
        self.value = unicode(value)


class Gateway(object):
    API_URL = u"https://api.ewaypayments.com"
    SANDBOX_URL = u"https://api.sandbox.ewaypayments.com"

    def __init__(self, api_key, password, use_sandbox=False):
        self.base_url = self.API_URL
        self.api_key = api_key
        if not self.api_key:
            raise ImproperlyConfigured(
                "API key for eWay required but can't find EWAY_API_KEY "
                "in setting. Please check and try again.")

        self.password = password
        if not self.password:
            raise ImproperlyConfigured(
                "API key for eWay required but can't find EWAY_PASSWORD "
                "in setting. Please check and try again.")

        if settings.EWAY_USE_SANDBOX:
            self.base_url = self.SANDBOX_URL

    def process_payment(self):
        raise NotImplementedError()

    def token_payment(self, order_number, total_incl_tax, redirect_url,
                      billing_address, **kwargs):

        eway_request = Request(
            method=EWAY_TOKEN_PAYMENT,
            redirect_url=redirect_url,
            payment=Payment(
                total_amount=total_incl_tax,
                # use invoice reference rather then invoice number because
                # a 16-digit restriction might be too tough
                invoice_reference=order_number,
            ),
            customer=Customer(
                billing_address=billing_address,
                **kwargs
            ),
            customer_ip=kwargs.get('customer_ip', None),
            device_id=kwargs.get('device_id', None),
            #FIXME add the shipping address properly
            #shipping_address=kwargs.get('shipping_address', None),
            items=kwargs.get('items', None),
            options=kwargs.get('options', None),
        )

        response = self.access_codes(eway_request)
        return response

    def create_token_customer(self, title, first_name, last_name,
                              country_code):
        raise NotImplementedError()

    def update_token_customer(self):
        raise NotImplementedError()

    def access_codes(self, request):
        url = "%s/AccessCodes" % self.base_url
        request_json = request.serialise()

        logger.info("requesting new access code: {0}".format(url))

        transaction = Transaction.objects.create(access_code='')
        request_log = RequestLog.objects.create(
            transaction=transaction, request=request_json)

        response = self._post(url, data=request_json)
        response = RapidResponse.from_json(response.json())

        transaction.access_code = response.access_code
        transaction.amount = request.payment.total_amount

        transaction.order_number = request.payment.invoice_reference
        transaction.save()

        request_log.method = unicode(request.method)
        request_log.url = url
        request_log.request = json.dumps(request.json(), indent=4)
        request_log.response = response.json_raw

        errors = {}
        for error in response.errors:
            errors[error.code] = error.message

        request_log.errors = json.dumps(errors)
        request_log.save()

        return response

    def get_access_code_result(self, access_code):
        request_url = "%s/AccessCode/%s" % (self.base_url, access_code)

        logger.info(
            'requesting results for access code: {0}'.format(access_code))

        transaction, __ = Transaction.objects.get_or_create(
            access_code=access_code)

        request_log = RequestLog.objects.create(
            transaction=transaction, request='{}')

        response = self._get(request_url)
        response = RapidAccessCodeResult.from_json(response.json())

        logger.info("results for access code '{0}': {1}".format(
            access_code, response.json_raw))

        transaction.order_number = response.invoice_reference
        transaction.amount = response.total_amount

        if response.transaction_id:
            transaction.transaction_id = response.transaction_id

        if response.token_customer_id:
            transaction.token_customer_id = response.token_customer_id

        transaction.status = transaction.COMPLETED
        transaction.save()

        request_log.method = "GetAccessCodeResult"
        request_log.url = request_url
        request_log.response = response.json_raw

        errors = {}
        for error in response.errors:
            errors[error.code] = error.message

        request_log.errors = json.dumps(errors)
        request_log.response_code = response.response_code.code
        request_log.response_message = response.response_code.message

        request_log.save()

        return response

    def _get(self, url):
        try:
            response = requests.get(url, auth=(self.api_key, self.password))
        except Exception as exc:
            logger.error("could not connect to eWay server: {0}".format(url))
            raise RapidError(
                'problem connecting to eWay server: %s' % (exc.message))

        if response.status_code != 200:
            logger.error("eWay responded with error {0}: {1}".format(
                response.status_code, response.reason))
            raise RapidError(
                'received error response: [%s] %s' % (
                    response.status_code, response.reason))
        return response

    def _post(self, url, data):
        headers = {'content-type': 'application/json'}
        try:
            response = requests.post(
                "%s/AccessCodes" % self.base_url,
                auth=(self.api_key, self.password), data=data, headers=headers)
        except Exception as exc:
            logger.error(
                "could not connect to eWay server: {0}".format(url),
                extra={"data": data, 'headers': headers})
            raise RapidError(
                'problem connecting to eWay server: %s' % (exc.message))

        if response.status_code != 200:
            logger.error("eWay responded with error {0}: {1}".format(
                response.status_code, response.reason),
                extra={"data": data, 'headers': headers})
            raise RapidError(
                'received error response: [%s] %s' % (
                    response.status_code, response.reason))
        return response



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

TRANSACTION_APPROVED = '00'

TRANSACTION_RESPONSE_CODES = {
    TRANSACTION_APPROVED: 'Transaction Approved',
    '01': 'Refer to Issuer',
    '02': 'Refer to Issuer, special',
    '03': 'No Merchant',
    '04': 'Pick Up Card',
    '05': 'Do Not Honour',
    '06': 'Error',
    '07': 'Pick Up Card, Special',
    '08': 'Honour With Identification',
    '09': 'Request In Progress',
    '10': 'Approved For Partial Amount',
    '11': 'Approved, VIP',
    '12': 'Invalid Transaction',
    '13': 'Invalid Amount',
    '14': 'Invalid Card Number',
    '15': 'No Issuer',
    '16': 'Approved, Update Track 3',
    '19': 'Re-enter Last Transaction',
    '21': 'No Action Taken',
    '22': 'Suspected Malfunction',
    '23': 'Unacceptable Transaction Fee',
    '25': 'Unable to Locate Record On File',
    '30': 'Format Error',
    '31': 'Bank Not Supported By Switch',
    '33': 'Expired Card, Capture',
    '34': 'Suspected Fraud, Retain Card',
    '35': 'Card Acceptor, Contact Acquirer, Retain Card',
    '36': 'Restricted Card, Retain Card',
    '37': 'Contact Acquirer Security Department, Retain Card',
    '38': 'PIN Tries Exceeded, Capture',
    '39': 'No Credit Account',
    '40': 'Function Not Supported',
    '41': 'Lost Card',
    '42': 'No Universal Account',
    '43': 'Stolen Card',
    '44': 'No Investment Account',
    '51': 'Insufficient Funds',
    '52': 'No Cheque Account',
    '53': 'No Savings Account',
    '54': 'Expired Card',
    '55': 'Incorrect PIN',
    '56': 'No Card Record',
    '57': 'Function Not Permitted to Cardholder',
    '58': 'Function Not Permitted to Terminal',
    '59': 'Suspected Fraud',
    '60': 'Acceptor Contact Acquirer',
    '61': 'Exceeds Withdrawal Limit',
    '62': 'Restricted Card',
    '63': 'Security Violation',
    '64': 'Original Amount Incorrect',
    '66': 'Acceptor Contact Acquirer, Security',
    '67': 'Capture Card',
    '75': 'PIN Tries Exceeded',
    '82': 'CVV Validation Error',
    '84': 'Do Not Honour (Sandbox',
    '90': 'Cutoff In Progress',
    '91': 'Card Issuer Unavailable',
    '92': 'Unable To Route Transaction',
    '93': 'Cannot Complete, Violation Of The Law',
    '94': 'Duplicate Transaction',
    '96': 'System Error',
}

RESPONSE_CODES = {
    # Transaction response codes
    'A2000': 'Transaction Approved Successful',
    'A2008': 'Honour With Identification Successful',
    'A2010': 'Approved For Partial Amount Successful',
    'A2011': 'Approved, VIP Successful',
    'A2016': 'Approved, Update Track 3 Successful',
    'D4401': 'Refer to Issuer Failed',
    'D4402': 'Refer to Issuer, special Failed',
    'D4403': 'No Merchant Failed',
    'D4404': 'Pick Up Card Failed',
    'D4405': 'Do Not Honour Failed',
    'D4406': 'Error Failed',
    'D4407': 'Pick Up Card, Special Failed',
    'D4409': 'Request In Progress Failed',
    'D4412': 'Invalid Transaction Failed',
    'D4413': 'Invalid Amount Failed',
    'D4414': 'Invalid Card Number Failed',
    'D4415': 'No Issuer Failed',
    'D4419': 'Re-enter Last Transaction Failed',
    'D4421': 'No Action Taken Failed',
    'D4422': 'Suspected Malfunction Failed',
    'D4423': 'Unacceptable Transaction Fee Failed',
    'D4425': 'Unable to Locate Record On File Failed',
    'D4430': 'Format Error Failed',
    'D4431': 'Bank Not Supported By Switch Failed',
    'D4433': 'Expired Card, Capture Failed',
    'D4434': 'Suspected Fraud, Retain Card Failed',
    'D4435': 'Card Acceptor, Contact Acquirer, Retain Card Failed',
    'D4436': 'Restricted Card, Retain Card Failed',
    'D4437': 'Contact Acquirer Security Department, Retain Card Failed',
    'D4438': 'PIN Tries Exceeded, Capture Failed',
    'D4439': 'No Credit Account Failed',
    'D4440': 'Function Not Supported Failed',
    'D4441': 'Lost Card Failed',
    'D4442': 'No Universal Account Failed',
    'D4443': 'Stolen Card Failed',
    'D4444': 'No Investment Account Failed',
    'D4451': 'Insufficient Funds Failed',
    'D4452': 'No Cheque Account Failed',
    'D4453': 'No Savings Account Failed',
    'D4454': 'Expired Card Failed',
    'D4455': 'Incorrect PIN Failed',
    'D4456': 'No Card Record Failed',
    'D4457': 'Function Not Permitted to Cardholder Failed',
    'D4458': 'Function Not Permitted to Terminal Failed',
    'D4459': 'Suspected Fraud Failed',
    'D4460': 'Acceptor Contact Acquirer Failed',
    'D4461': 'Exceeds Withdrawal Limit Failed',
    'D4462': 'Restricted Card Failed',
    'D4463': 'Security Violation Failed',
    'D4464': 'Original Amount Incorrect Failed',
    'D4466': 'Acceptor Contact Acquirer, Security Failed',
    'D4467': 'Capture Card Failed',
    'D4475': 'PIN Tries Exceeded Failed',
    'D4482': 'CVV Validation Error Failed',
    'D4490': 'Cut off In Progress Failed',
    'D4491': 'Card Issuer Unavailable Failed',
    'D4492': 'Unable To Route Transaction Failed',
    'D4493': 'Cannot Complete, Violation Of The Law Failed',
    'D4494': 'Duplicate Transaction Failed',
    'D4496': 'System Error Failed',
    # Beagle (Free and Beagle Alerts Fraud response
    'F7000': 'Undefined Fraud Error',
    'F7001': 'Challenged Fraud',
    'F7002': 'Country Match Fraud',
    'F7003': 'High Risk Country Fraud',
    'F7004': 'Anonymous Proxy Fraud',
    'F7005': 'Transparent Proxy Fraud',
    'F7006': 'Free Email Fraud',
    'F7007': 'International Transaction Fraud',
    'F7008': 'Risk Score Fraud',
    'F7009': 'Denied Fraud',
    'F9010': 'High Risk Billing Country',
    'F9011': 'High Risk Credit Card Country',
    'F9012': 'High Risk Customer IP Address',
    'F9013': 'High Risk Email Address',
    'F9014': 'High Risk Shipping Country',
    'F9015': 'Multiple card numbers for single email address',
    'F9016': 'Multiple card numbers for single location',
    'F9017': 'Multiple email addresses for single card number',
    'F9018': 'Multiple email addresses for single location',
    'F9019': 'Multiple locations for single card number',
    'F9020': 'Multiple locations for single email address',
    'F9021': 'Suspicious Customer First Name',
    'F9022': 'Suspicious Customer Last Name',
    'F9023': 'Transaction Declined',
    'F9024': 'Multiple transactions for same address with known credit card',
    'F9025': 'Multiple transactions for same address with new credit card',
    'F9026': 'Multiple transactions for same email with new credit card',
    'F9027': 'Multiple transactions for same email with known credit card',
    'F9028': 'Multiple transactions for new credit card',
    'F9029': 'Multiple transactions for known credit card',
    'F9030': 'Multiple transactions for same email address',
    'F9031': 'Multiple transactions for same credit card',
    'F9032': 'Invalid Customer Last Name',
    'F9033': 'Invalid Billing Street',
    'F9034': 'Invalid Shipping Street',
    # System response codes
    'S5000': 'System Error',
    'S5085': 'Started 3dSecure',
    'S5086': 'Routed 3dSecure',
    'S5087': 'Completed 3dSecure',
    'S5099': 'Incomplete (Access Code in progress/incomplete',
    # Validation response codes
    'V6000': 'Validation error',
    'V6001': 'Invalid CustomerIP',
    'V6002': 'Invalid DeviceID',
    'V6011': 'Invalid Payment TotalAmount',
    'V6012': 'Invalid Payment InvoiceDescription',
    'V6013': 'Invalid Payment InvoiceNumber',
    'V6014': 'Invalid Payment InvoiceReference',
    'V6015': 'Invalid Payment CurrencyCode',
    'V6016': 'Payment Required',
    'V6017': 'Payment CurrencyCode Required',
    'V6018': 'Unknown Payment CurrencyCode',
    'V6021': 'EWAY_CARDHOLDERNAME Required',
    'V6022': 'EWAY_CARDNUMBER Required',
    'V6023': 'EWAY_CARDCVN Required',
    'V6033': 'Invalid Expiry Date',
    'V6034': 'Invalid Issue Number',
    'V6035': 'Invalid Valid From Date',
    'V6040': 'Invalid TokenCustomerID',
    'V6041': 'Customer Required',
    'V6042': 'Customer FirstName Required',
    'V6043': 'Customer LastName Required',
    'V6044': 'Customer CountryCode Required',
    'V6045': 'Customer Title Required',
    'V6046': 'TokenCustomerID Required',
    'V6047': 'RedirectURL Required',
    'V6051': 'Invalid Customer FirstName',
    'V6052': 'Invalid Customer LastName',
    'V6053': 'Invalid Customer CountryCode',
    'V6058': 'Invalid Customer Title',
    'V6059': 'Invalid RedirectURL',
    'V6060': 'Invalid TokenCustomerID',
    'V6061': 'Invalid Customer Reference',
    'V6062': 'Invalid Customer CompanyName',
    'V6063': 'Invalid Customer JobDescription',
    'V6064': 'Invalid Customer Street1',
    'V6065': 'Invalid Customer Street2',
    'V6066': 'Invalid Customer City',
    'V6067': 'Invalid Customer State',
    'V6068': 'Invalid Customer PostalCode',
    'V6069': 'Invalid Customer Email',
    'V6070': 'Invalid Customer Phone',
    'V6071': 'Invalid Customer Mobile',
    'V6072': 'Invalid Customer Comments',
    'V6073': 'Invalid Customer Fax',
    'V6074': 'Invalid Customer URL',
    'V6075': 'Invalid ShippingAddress FirstName',
    'V6076': 'Invalid ShippingAddress LastName',
    'V6077': 'Invalid ShippingAddress Street1',
    'V6078': 'Invalid ShippingAddress Street2',
    'V6079': 'Invalid ShippingAddress City',
    'V6080': 'Invalid ShippingAddress State',
    'V6081': 'Invalid ShippingAddress PostalCode',
    'V6082': 'Invalid ShippingAddress Email',
    'V6083': 'Invalid ShippingAddress Phone',
    'V6084': 'Invalid ShippingAddress Country',
    'V6085': 'Invalid ShippingAddress ShippingMethod',
    'V6086': 'Invalid ShippingAddress Fax',
    'V6091': 'Unknown Customer CountryCode',
    'V6092': 'Unknown ShippingAddress CountryCode',
    'V6100': 'Invalid EWAY_CARDNAME',
    'V6101': 'Invalid EWAY_CARDEXPIRYMONTH',
    'V6102': 'Invalid EWAY_CARDEXPIRYYEAR',
    'V6103': 'Invalid EWAY_CARDSTARTMONTH',
    'V6104': 'Invalid EWAY_CARDSTARTYEAR',
    'V6105': 'Invalid EWAY_CARDISSUENUMBER',
    'V6106': 'Invalid EWAY_CARDCVN',
    'V6107': 'Invalid EWAY_ACCESSCODE',
    'V6108': 'Invalid CustomerHostAddress',
    'V6109': 'Invalid UserAgent',
    'V6110': 'Invalid EWAY_CARDNUMBER',
}
RESPONSE_CODES.update(TRANSACTION_RESPONSE_CODES)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect

from oscar.core.loading import get_class

from . import forms
from . import gateway
from .facade import Facade
from .. import compat
from .. import PAYMENT_METHOD_EWAY

OrderNumberGenerator = get_class('order.utils', 'OrderNumberGenerator')
PaymentError = get_class('payment.exceptions', 'PaymentError')
CheckoutSessionData = get_class('checkout.utils', 'CheckoutSessionData')


class EwayPaymentDetailMixin(object):
    facade = Facade()

    def get_context_data(self, **kwargs):
        ctx = super(EwayPaymentDetailMixin, self).get_context_data(**kwargs)

        if self.preview:
            bankcard_form = forms.BankcardForm(self.request.POST)
        else:
            bankcard_form = forms.BankcardForm()

        ctx['bankcard_form'] = kwargs.get('bankcard_form', bankcard_form)
        ctx['payment_method'] = self.checkout_session.payment_method()

        if not 'form_action_url' in ctx:
            ctx['form_action_url'] = reverse('checkout:preview')

        return ctx

    def post(self, request, *args, **kwargs):
        """
        This method is designed to be overridden by subclasses which will
        validate the forms from the payment details page. If the forms are
        valid then the method can call submit().
        """
        error_response = self.get_error_response()
        if error_response:
            return error_response

        if self.preview:
            self.checkout_session.pay_by(PAYMENT_METHOD_EWAY)

            # We use a custom parameter to indicate if this is an attempt to
            # place an order. Without this, we assume a payment form is being
            # submitted from the payment-details page.
            if request.POST.get('action', '') == 'place_order':
                if compat.USES_NEW_STRATEGY_PATTERN:
                    return self.submit(**self.build_submission())
                else:
                    return self.submit(request.basket)

            return self.render_eway_preview(request)

        # Posting to payment-details isn't the right thing to do
        return self.get(request, *args, **kwargs)

    def render_eway_preview(self, request, **kwargs):
        try:
            response = self.get_eway_access_code(request.basket)
        except gateway.RapidError:
            messages.error(
                self.request,
                "invalid payment details. please try again"
            )
            return HttpResponseRedirect(reverse('checkout:payment-details'))

        data = self.request.POST.copy()
        data['EWAY_ACCESSCODE'] = response.access_code
        return self.render_preview(
            request, form_action_url=response.form_action_url,
            bankcard_form=forms.BankcardForm(data))

    def get_eway_access_code(self, basket):
        order_number = self.generate_order_number(basket)
        if compat.USES_NEW_STRATEGY_PATTERN:
            total = self.get_order_totals(
                basket, self.get_shipping_method(basket))
            total_incl_tax = total.incl_tax
        else:
            total_incl_tax, __ = self.get_order_totals(basket)

        try:
            billing_address = self.get_billing_address()
        except AttributeError:
            billing_address = self.get_default_billing_address()

        if compat.USES_NEW_STRATEGY_PATTERN:
            shipping_address = self.get_shipping_address(basket)
        else:
            shipping_address = self.get_shipping_address()

        billing_address = billing_address or shipping_address

        redirect_url = "http://%s%s" % (
            self.request.META['HTTP_HOST'], reverse('eway-rapid-response'))

        response = self.facade.token_payment(
            order_number, total_incl_tax, redirect_url=redirect_url,
            billing_address=billing_address, shipping_address=shipping_address)
        return response

    def generate_order_number(self, basket):
        if self.checkout_session.get_order_number():
            return self.checkout_session.get_order_number()

        generator = OrderNumberGenerator()
        order_number = generator.order_number(basket)
        self.checkout_session.set_order_number(order_number)
        return order_number



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.conf.urls import url

from eway.rapid import views


urlpatterns = [
    url(r'^preview/$',
        views.RapidResponseView.as_view(preview=True),
        name='eway-rapid-response'),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

__version__ = VERSION = (0, 1, 0)

PURCHASE = 'Purchase'
PAYMENT_METHOD_EWAY = "eWay"


def get_version():
    return '.'.join([str(i) for i in VERSION[:3]])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from datetime import datetime

from django.conf import settings
from django.contrib import messages
from django.db.models import get_model
from django.utils.translation import ugettext_lazy as _
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse

from oscar.core.loading import get_class

from .facade import Facade
from .. import compat
from .. import PURCHASE, PAYMENT_METHOD_EWAY


BankcardModel = get_model('payment', 'Bankcard')
PaymentDetailsView = get_class('checkout.views', 'PaymentDetailsView')

PaymentError = get_class('payment.exceptions', 'PaymentError')
SourceType = get_model('payment', 'SourceType')
Source = get_model('payment', 'Source')


class RapidResponseView(PaymentDetailsView):
    facade = Facade()

    def get(self, request, *args, **kwargs):
        self.access_code = request.GET.get('AccessCode', None)

        if compat.USES_NEW_STRATEGY_PATTERN:
            return self.submit(**self.build_submission())
        else:
            return self.submit(request.basket)

    def get_bankcard_details(self):
        kwargs = self.checkout_session._get('bankcard', 'bankcard_fields')
        if not kwargs:
            return {}
        expiry_date = datetime.strptime(kwargs.get('expiry_date'), "%m/%Y").date()
        kwargs['expiry_date'] = expiry_date
        return kwargs

    def set_bankcard_details(self, bankcard_kwargs):
        self.checkout_session._unset('bankcard', 'bankcard_fields')
        self.checkout_session._set('bankcard', 'bankcard_fields', bankcard_kwargs)

    def store_bankcard(self, token_customer_id):
        kwargs = self.get_bankcard_details()

        if not kwargs:
            return

        kwargs.update({
            'user': self.request.user,
            'partner_reference': token_customer_id,
        })

        try:
            bankcard = BankcardModel.objects.get(
                user=self.request.user,
                partner_reference=token_customer_id,
            )
        except BankcardModel.DoesNotExist:
            bankcard = BankcardModel.objects.create(**kwargs)
        else:
            BankcardModel.objects.filter(pk=bankcard.id).update(**kwargs)

    def handle_payment(self, order_number, total, **kwargs):
        response = self.facade.get_access_code_result(self.access_code)

        # this is required because the new strategy pattern introduced in
        # Oscar 0.6 changes the data passed into this method. This is purely
        # for backwards-compatibility
        try:
            total_incl_tax = total.incl_tax
        except AttributeError:
            total_incl_tax = total

        if not response.transaction_status:
            messages.error(
                self.request,
                _("We experienced a problem while processing your payment. "
                  "Please check your card details and try again. If the "
                  "problem persists, please contact us."))
            raise PaymentError(
                "received error(s) '%s' from eway for transaction #%s" % (
                    [(c.code, c.message) for c in response.response_message],
                    response.transaction_id,
                )
            )

        if self.request.user.is_authenticated():
            self.store_bankcard(response.token_customer_id)

        # Request was successful - record the "payment source".  As this
        # request was a 'pre-auth', we set the 'amount_allocated' - if we had
        # performed an 'auth' request, then we would set 'amount_debited'.
        source_type, __ = SourceType.objects.get_or_create(
            name=PAYMENT_METHOD_EWAY
        )
        source = Source(
            source_type=source_type,
            currency=settings.EWAY_CURRENCY,
            amount_allocated=total_incl_tax,
            amount_debited=total_incl_tax,
            reference=response.transaction_id
        )
        self.add_payment_source(source)

        # Also record payment event
        self.add_payment_event(PURCHASE, total_incl_tax)

    def render_to_response(self, context, **response_kwargs):
        if 'error' in context:
            return HttpResponseRedirect(reverse('checkout:payment-details'))
        return super(RapidResponseView, self).render_to_response(
            context,
            **response_kwargs
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from django.conf import settings

from oscar.core.loading import get_classes

from eway.rapid import gateway
from eway import PURCHASE  # noqa


(RedirectRequired, UnableToTakePayment, PaymentError) = get_classes(
    'payment.exceptions',
    ['RedirectRequired', 'UnableToTakePayment', 'PaymentError'])


def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


class Facade(object):

    def __init__(self):
        self.gateway = gateway.Gateway(
            getattr(settings, 'EWAY_API_KEY', gateway.EWAY_API_KEY),
            getattr(settings, 'EWAY_PASSWORD', gateway.EWAY_PASSWORD))

        self.currency = getattr(
            settings, 'EWAY_CURRENCY', gateway.EWAY_CURRENCY)

    def token_payment(self, order_number, total_incl_tax, redirect_url,
                      billing_address, token_customer_id=None,
                      shipping_address=None, customer_ip=None,
                      basket=None, options=None, **kwargs):
        #TODO extract items from basket

        response = self.gateway.token_payment(
            order_number=order_number,
            total_incl_tax=total_incl_tax,
            redirect_url=redirect_url,
            token_customer_id=token_customer_id,
            customer_ip=customer_ip,
            billing_address=billing_address,
            #TODO We need to get the actual shipping address created
            # and provide it to the token_payment method
            #NOTE this is the shipping address used in Rapid NOT the
            # Oscar shipping address
            shipping_address=shipping_address,
            #TODO We need to get the actual basket items created
            # and provide it to the token_payment method
            items=None,
            options=options,
            **kwargs)
        return response

    def get_access_code_result(self, access_code):
        return self.gateway.get_access_code_result(access_code)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

from eway import get_version


setup(
    name='django-oscar-eway',
    version=get_version(),
    url='https://github.com/tangentlabs/django-oscar-eway',
    author="Sebastian Vetter",
    author_email="sebastian.vetter@tangentsnowball.com.au",
    description="eWay payment module for django-oscar (Rapid 3.0)",
    long_description='\n\n'.join([
        open('README.rst').read(),
        open('CHANGELOG.rst').read(),
    ]),
    keywords="oscar,payment,django,ecommerce,eway,rapid3.0",
    license='BSD',
    packages=find_packages(exclude=['sandbox*', 'tests*']),
    install_requires=[
        'django-oscar>=0.4',
        'requests>=1.0',
    ],
    include_package_data=True,
    # See http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python'
    ]
)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        db.rename_column('eway_ewaytransaction', 'txn_ref', 'transaction_id')
        db.rename_column('eway_ewaytransaction', 'txn_url', 'request_url')
        db.rename_column('eway_ewaytransaction', 'txn_method', 'method')

    def backwards(self, orm):
        db.rename_column('eway_ewaytransaction', 'transaction_id', 'txn_ref')
        db.rename_column('eway_ewaytransaction', 'request_url', 'txn_url')
        db.rename_column('eway_ewaytransaction', 'method', 'txn_method')

    models = {
        'eway.ewayresponsecode': {
            'Meta': {'object_name': 'EwayResponseCode'},
            'code': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '10'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'message': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'transactions': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'response_messages'", 'symmetrical': 'False', 'to': "orm['eway.EwayTransaction']"})
        },
        'eway.ewaytransaction': {
            'Meta': {'ordering': "('-date_created',)", 'object_name': 'EwayTransaction'},
            'amount': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '12', 'decimal_places': '2', 'blank': 'True'}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'method': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True'}),
            'order_number': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'db_index': 'True'}),
            'request_json': ('django.db.models.fields.TextField', [], {}),
            'request_url': ('django.db.models.fields.CharField', [], {'max_length': '800', 'null': 'True'}),
            'response_code': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True'}),
            'response_json': ('django.db.models.fields.TextField', [], {'null': 'True'}),
            'response_message': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True'}),
            'token_customer_id': ('django.db.models.fields.CharField', [], {'max_length': '16', 'null': 'True', 'db_index': 'True'}),
            'transaction_id': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True'})
        }
    }

    complete_apps = ['eway']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from django.conf.urls import url

from . import views


urlpatterns = [
    url(r'^eway/$',
        views.TransactionListView.as_view(), name='transaction-list'),
    url(r'^eway/transaction/(?P<pk>\d+)/$',
        views.TransactionDetailView.as_view(), name='transaction-detail'),
]



# -*- coding: utf-8 -*-
import json
import datetime

from south.db import db
from south.v2 import DataMigration
from django.db import models
from django.core.exceptions import ObjectDoesNotExist


def convert_deprecated_transaction(transaction, orm):
    try:
        response = json.loads(transaction.response_json)
    except (ValueError, TypeError):
        response = {}
    access_code = response.get('AccessCode') or ''

    try:
        new_transaction = orm.EwayTransaction.objects.get(
            access_code=access_code)
    except ObjectDoesNotExist:
        new_transaction = orm.EwayTransaction.objects.create(
            access_code=access_code,
            transaction_id=transaction.transaction_id or '',
            order_number=transaction.order_number or '',
            token_customer_id=transaction.token_customer_id or '',
            amount=transaction.amount,
            date_created=transaction.date_created,
            date_modified=transaction.date_created)

    request_errors = {}
    for error in transaction.response_messages.all():
        request_errors[error.code] = error.message

    orm.RequestLog.objects.get_or_create(
        transaction=new_transaction,
        url=transaction.request_url,
        method=transaction.method,
        request=transaction.request_json,
        response=transaction.response_json,
        response_code=transaction.response_code or '',
        response_message=transaction.response_message or '',
        errors=json.dumps(request_errors),
        date_created=transaction.date_created,
        date_modified=transaction.date_created)


class Migration(DataMigration):

    def forwards(self, orm):
        "Write your forwards methods here."
        for transaction in orm.Transaction.objects.all():
            convert_deprecated_transaction(transaction, orm)

    def backwards(self, orm):
        "Write your backwards methods here."
        raise RuntimeError('converting transactions cannot be reverted')

    models = {
        'address.country': {
            'Meta': {'ordering': "('-is_highlighted', 'name')", 'object_name': 'Country'},
            'is_highlighted': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'is_shipping_country': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'iso_3166_1_a2': ('django.db.models.fields.CharField', [], {'max_length': '2', 'primary_key': 'True'}),
            'iso_3166_1_a3': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'db_index': 'True'}),
            'iso_3166_1_numeric': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'db_index': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'printable_name': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'basket.basket': {
            'Meta': {'object_name': 'Basket'},
            'date_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date_merged': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'date_submitted': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'baskets'", 'null': 'True', 'to': "orm['auth.User']"}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'Open'", 'max_length': '128'}),
            'vouchers': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['voucher.Voucher']", 'null': 'True', 'blank': 'True'})
        },
        'catalogue.attributeentity': {
            'Meta': {'object_name': 'AttributeEntity'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255', 'blank': 'True'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'entities'", 'to': "orm['catalogue.AttributeEntityType']"})
        },
        'catalogue.attributeentitytype': {
            'Meta': {'object_name': 'AttributeEntityType'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255', 'blank': 'True'})
        },
        'catalogue.attributeoption': {
            'Meta': {'object_name': 'AttributeOption'},
            'group': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'options'", 'to': "orm['catalogue.AttributeOptionGroup']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'option': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'catalogue.attributeoptiongroup': {
            'Meta': {'object_name': 'AttributeOptionGroup'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        'catalogue.category': {
            'Meta': {'ordering': "['full_name']", 'object_name': 'Category'},
            'depth': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'full_name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'}),
            'numchild': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'path': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255'})
        },
        'catalogue.option': {
            'Meta': {'object_name': 'Option'},
            'code': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '128'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'type': ('django.db.models.fields.CharField', [], {'default': "'Required'", 'max_length': '128'})
        },
        'catalogue.product': {
            'Meta': {'ordering': "['-date_created']", 'object_name': 'Product'},
            'attributes': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['catalogue.ProductAttribute']", 'through': "orm['catalogue.ProductAttributeValue']", 'symmetrical': 'False'}),
            'categories': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['catalogue.Category']", 'through': "orm['catalogue.ProductCategory']", 'symmetrical': 'False'}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'date_updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'db_index': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_discountable': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'variants'", 'null': 'True', 'to': "orm['catalogue.Product']"}),
            'product_class': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['catalogue.ProductClass']", 'null': 'True'}),
            'product_options': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['catalogue.Option']", 'symmetrical': 'False', 'blank': 'True'}),
            'rating': ('django.db.models.fields.FloatField', [], {'null': 'True'}),
            'recommended_products': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['catalogue.Product']", 'symmetrical': 'False', 'through': "orm['catalogue.ProductRecommendation']", 'blank': 'True'}),
            'related_products': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'relations'", 'blank': 'True', 'to': "orm['catalogue.Product']"}),
            'score': ('django.db.models.fields.FloatField', [], {'default': '0.0', 'db_index': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '255'}),
            'status': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'upc': ('django.db.models.fields.CharField', [], {'max_length': '64', 'unique': 'True', 'null': 'True', 'blank': 'True'})
        },
        'catalogue.productattribute': {
            'Meta': {'ordering': "['code']", 'object_name': 'ProductAttribute'},
            'code': ('django.db.models.fields.SlugField', [], {'max_length': '128'}),
            'entity_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['catalogue.AttributeEntityType']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'option_group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['catalogue.AttributeOptionGroup']", 'null': 'True', 'blank': 'True'}),
            'product_class': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'attributes'", 'null': 'True', 'to': "orm['catalogue.ProductClass']"}),
            'required': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'type': ('django.db.models.fields.CharField', [], {'default': "'text'", 'max_length': '20'})
        },
        'catalogue.productattributevalue': {
            'Meta': {'object_name': 'ProductAttributeValue'},
            'attribute': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['catalogue.ProductAttribute']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'product': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'attribute_values'", 'to': "orm['catalogue.Product']"}),
            'value_boolean': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'value_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'value_entity': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['catalogue.AttributeEntity']", 'null': 'True', 'blank': 'True'}),
            'value_float': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'value_integer': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'value_option': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['catalogue.AttributeOption']", 'null': 'True', 'blank': 'True'}),
            'value_richtext': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'value_text': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        'catalogue.productcategory': {
            'Meta': {'ordering': "['-is_canonical']", 'object_name': 'ProductCategory'},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['catalogue.Category']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_canonical': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'product': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['catalogue.Product']"})
        },
        'catalogue.productclass': {
            'Meta': {'ordering': "['name']", 'object_name': 'ProductClass'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'options': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['catalogue.Option']", 'symmetrical': 'False', 'blank': 'True'}),
            'requires_shipping': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '128'}),
            'track_stock': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        'catalogue.productrecommendation': {
            'Meta': {'object_name': 'ProductRecommendation'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'primary': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'primary_recommendations'", 'to': "orm['catalogue.Product']"}),
            'ranking': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '0'}),
            'recommendation': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['catalogue.Product']"})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'eway.ewaytransaction': {
            'Meta': {'ordering': "(u'-date_created',)", 'object_name': 'EwayTransaction'},
            'access_code': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'amount': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '12', 'decimal_places': '2', 'blank': 'True'}),
            'basket': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'eway_transactions'", 'null': 'True', 'on_delete': 'models.SET_NULL', 'to': "orm['basket.Basket']"}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {}),
            'date_modified': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'order_number': ('django.db.models.fields.CharField', [], {'default': 'u""', 'max_length': '255', 'blank': 'True'}),
            'token_customer_id': ('django.db.models.fields.CharField', [], {'default': 'u""', 'max_length': '16', 'blank': 'True'}),
            'transaction_id': ('django.db.models.fields.CharField', [], {'default': 'u""', 'max_length': '100', 'blank': 'True'})
        },
        'eway.requestlog': {
            'Meta': {'ordering': "(u'-date_created',)", 'object_name': 'RequestLog'},
            'date_created': ('django.db.models.fields.DateTimeField', [], {}),
            'date_modified': ('django.db.models.fields.DateTimeField', [], {}),
            'errors': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'method': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'request': ('django.db.models.fields.TextField', [], {}),
            'response': ('django.db.models.fields.TextField', [], {'default': 'u""', 'blank': 'True'}),
            'response_code': ('django.db.models.fields.CharField', [], {'default': 'u""', 'max_length': '2', 'blank': 'True'}),
            'response_message': ('django.db.models.fields.CharField', [], {'default': 'u""', 'max_length': '255', 'blank': 'True'}),
            'transaction': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'request_logs'", 'to': "orm['eway.EwayTransaction']"}),
            'url': ('django.db.models.fields.TextField', [], {'default': 'u""'})
        },
        'eway.responsecode': {
            'Meta': {'object_name': 'ResponseCode'},
            'code': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '10'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'message': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'transactions': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "u'response_messages'", 'symmetrical': 'False', 'to': "orm['eway.Transaction']"})
        },
        'eway.transaction': {
            'Meta': {'ordering': "(u'-date_created',)", 'object_name': 'Transaction'},
            'amount': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '12', 'decimal_places': '2', 'blank': 'True'}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'method': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True'}),
            'order_number': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'db_index': 'True'}),
            'request_json': ('django.db.models.fields.TextField', [], {}),
            'request_url': ('django.db.models.fields.CharField', [], {'max_length': '800', 'null': 'True'}),
            'response_code': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True'}),
            'response_json': ('django.db.models.fields.TextField', [], {'null': 'True'}),
            'response_message': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True'}),
            'token_customer_id': ('django.db.models.fields.CharField', [], {'max_length': '16', 'null': 'True', 'db_index': 'True'}),
            'transaction_id': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True'})
        },
        'offer.benefit': {
            'Meta': {'object_name': 'Benefit'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'max_affected_items': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'proxy_class': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '255', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'range': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['offer.Range']", 'null': 'True', 'blank': 'True'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'value': ('oscar.models.fields.PositiveDecimalField', [], {'null': 'True', 'max_digits': '12', 'decimal_places': '2', 'blank': 'True'})
        },
        'offer.condition': {
            'Meta': {'object_name': 'Condition'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'proxy_class': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '255', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'range': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['offer.Range']", 'null': 'True', 'blank': 'True'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'value': ('oscar.models.fields.PositiveDecimalField', [], {'null': 'True', 'max_digits': '12', 'decimal_places': '2', 'blank': 'True'})
        },
        'offer.conditionaloffer': {
            'Meta': {'ordering': "['-priority']", 'object_name': 'ConditionalOffer'},
            'benefit': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['offer.Benefit']"}),
            'condition': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['offer.Condition']"}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'end_datetime': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'max_basket_applications': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'max_discount': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '12', 'decimal_places': '2', 'blank': 'True'}),
            'max_global_applications': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'max_user_applications': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '128'}),
            'num_applications': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'num_orders': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'offer_type': ('django.db.models.fields.CharField', [], {'default': "'Site'", 'max_length': '128'}),
            'priority': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'redirect_url': ('oscar.models.fields.ExtendedURLField', [], {'max_length': '200', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '128', 'unique': 'True', 'null': 'True'}),
            'start_datetime': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'Open'", 'max_length': '64'}),
            'total_discount': ('django.db.models.fields.DecimalField', [], {'default': "'0.00'", 'max_digits': '12', 'decimal_places': '2'})
        },
        'offer.range': {
            'Meta': {'object_name': 'Range'},
            'classes': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'classes'", 'blank': 'True', 'to': "orm['catalogue.ProductClass']"}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'excluded_products': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'excludes'", 'blank': 'True', 'to': "orm['catalogue.Product']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'included_categories': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'includes'", 'blank': 'True', 'to': "orm['catalogue.Category']"}),
            'included_products': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'includes'", 'blank': 'True', 'to': "orm['catalogue.Product']"}),
            'includes_all_products': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '128'}),
            'proxy_class': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '255', 'unique': 'True', 'null': 'True', 'blank': 'True'})
        },
        'order.billingaddress': {
            'Meta': {'object_name': 'BillingAddress'},
            'country': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['address.Country']"}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'line1': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'line2': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'line3': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'line4': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'postcode': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'}),
            'search_text': ('django.db.models.fields.CharField', [], {'max_length': '1000'}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'})
        },
        'order.order': {
            'Meta': {'ordering': "['-date_placed']", 'object_name': 'Order'},
            'basket_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'billing_address': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['order.BillingAddress']", 'null': 'True', 'blank': 'True'}),
            'date_placed': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'db_index': 'True', 'blank': 'True'}),
            'guest_email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'number': ('django.db.models.fields.CharField', [], {'max_length': '128', 'db_index': 'True'}),
            'shipping_address': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['order.ShippingAddress']", 'null': 'True', 'blank': 'True'}),
            'shipping_excl_tax': ('django.db.models.fields.DecimalField', [], {'default': '0', 'max_digits': '12', 'decimal_places': '2'}),
            'shipping_incl_tax': ('django.db.models.fields.DecimalField', [], {'default': '0', 'max_digits': '12', 'decimal_places': '2'}),
            'shipping_method': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'total_excl_tax': ('django.db.models.fields.DecimalField', [], {'max_digits': '12', 'decimal_places': '2'}),
            'total_incl_tax': ('django.db.models.fields.DecimalField', [], {'max_digits': '12', 'decimal_places': '2'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'orders'", 'null': 'True', 'to': "orm['auth.User']"})
        },
        'order.shippingaddress': {
            'Meta': {'object_name': 'ShippingAddress'},
            'country': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['address.Country']"}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'line1': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'line2': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'line3': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'line4': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'notes': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'phone_number': ('django.db.models.fields.CharField', [], {'max_length': '32', 'null': 'True', 'blank': 'True'}),
            'postcode': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'}),
            'search_text': ('django.db.models.fields.CharField', [], {'max_length': '1000'}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'voucher.voucher': {
            'Meta': {'object_name': 'Voucher'},
            'code': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '128', 'db_index': 'True'}),
            'date_created': ('django.db.models.fields.DateField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'end_date': ('django.db.models.fields.DateField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'num_basket_additions': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'num_orders': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'offers': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'vouchers'", 'symmetrical': 'False', 'to': "orm['offer.ConditionalOffer']"}),
            'start_date': ('django.db.models.fields.DateField', [], {}),
            'total_discount': ('django.db.models.fields.DecimalField', [], {'default': "'0.00'", 'max_digits': '12', 'decimal_places': '2'}),
            'usage': ('django.db.models.fields.CharField', [], {'default': "'Multi-use'", 'max_length': '128'})
        }
    }

    complete_apps = ['eway']
    symmetrical = True
